/**
 * DevExtreme (ui/drawer/ui.drawer.js)
 * Version: 18.2.0 (build 18207)
 * Build date: Thu Jul 26 2018
 *
 * Copyright (c) 2012 - 2018 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
var _renderer = require("../../core/renderer");
var _renderer2 = _interopRequireDefault(_renderer);
var _events_engine = require("../../events/core/events_engine");
var _events_engine2 = _interopRequireDefault(_events_engine);
var _type = require("../../core/utils/type");
var _type2 = _interopRequireDefault(_type);
var _click = require("../../events/click");
var _click2 = _interopRequireDefault(_click);
var _translator = require("../../animation/translator");
var _translator2 = _interopRequireDefault(_translator);
var _dom = require("../../core/utils/dom");
var _hide_top_overlay = require("../../mobile/hide_top_overlay");
var _component_registrator = require("../../core/component_registrator");
var _component_registrator2 = _interopRequireDefault(_component_registrator);
var _extend = require("../../core/utils/extend");
var _ui = require("../widget/ui.widget");
var _ui2 = _interopRequireDefault(_ui);
var _empty_template = require("../widget/empty_template");
var _empty_template2 = _interopRequireDefault(_empty_template);
var _deferred = require("../../core/utils/deferred");
var _window = require("../../core/utils/window");
var _window2 = _interopRequireDefault(_window);
var _uiDrawerRenderingStrategy = require("./ui.drawer.rendering.strategy.push");
var _uiDrawerRenderingStrategy2 = _interopRequireDefault(_uiDrawerRenderingStrategy);
var _uiDrawerRenderingStrategy3 = require("./ui.drawer.rendering.strategy.persistent");
var _uiDrawerRenderingStrategy4 = _interopRequireDefault(_uiDrawerRenderingStrategy3);
var _uiDrawerRenderingStrategy5 = require("./ui.drawer.rendering.strategy.temporary");
var _uiDrawerRenderingStrategy6 = _interopRequireDefault(_uiDrawerRenderingStrategy5);
var _uiDrawerRendering = require("./ui.drawer.rendering.strategy");

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        "default": obj
    }
}
var DRAWER_CLASS = "dx-drawer";
var DRAWER_WRAPPER_CLASS = "dx-drawer-wrapper";
var DRAWER_MENU_CONTENT_CLASS = "dx-drawer-menu-content";
var DRAWER_CONTENT_CLASS = "dx-drawer-content";
var DRAWER_SHADER_CLASS = "dx-drawer-shader";
var INVISIBLE_STATE_CLASS = "dx-state-invisible";
var OPENED_STATE_CLASS = "dx-drawer-opened";
var ANONYMOUS_TEMPLATE_NAME = "content";
var Drawer = _ui2.default.inherit({
    _getDefaultOptions: function() {
        return (0, _extend.extend)(this.callBase(), {
            menuPosition: "left",
            menuVisible: false,
            minWidth: null,
            maxWidth: null,
            showShader: true,
            menuTemplate: "menu",
            contentTemplate: "content",
            mode: "push",
            showMode: "slide",
            animationEnabled: true,
            animationDuration: 400
        })
    },
    _getAnonymousTemplateName: function() {
        return ANONYMOUS_TEMPLATE_NAME
    },
    _init: function() {
        this.callBase();
        this._initStrategy();
        this.$element().addClass(DRAWER_CLASS);
        this._animations = [];
        this._deferredAnimate = void 0;
        this._initHideTopOverlayHandler()
    },
    _initStrategy: function() {
        var mode = this.option("mode");
        var Strategy = void 0;
        if ("push" === mode) {
            Strategy = _uiDrawerRenderingStrategy2.default
        }
        if ("persistent" === mode) {
            Strategy = _uiDrawerRenderingStrategy4.default
        }
        if ("temporary" === mode) {
            Strategy = _uiDrawerRenderingStrategy6.default
        }
        this._strategy = new Strategy(this)
    },
    _initHideTopOverlayHandler: function() {
        this._hideMenuHandler = this.hideMenu.bind(this)
    },
    _initTemplates: function() {
        this.callBase();
        this._defaultTemplates.menu = new _empty_template2.default(this);
        this._defaultTemplates.content = new _empty_template2.default(this)
    },
    _initMarkup: function() {
        this.callBase();
        this._togglePositionClass(this.option("menuVisible"));
        this._renderMarkup();
        this._refreshModeClass();
        this._refreshShowModeClass();
        var menuTemplate = this._getTemplate(this.option("menuTemplate"));
        menuTemplate && menuTemplate.render({
            container: this.menuContent()
        });
        var contentTemplateOption = this.option("contentTemplate"),
            contentTemplate = this._getTemplate(contentTemplateOption),
            transclude = this._getAnonymousTemplateName() === contentTemplateOption;
        contentTemplate && contentTemplate.render({
            container: this.content(),
            noModel: true,
            transclude: transclude
        });
        this._initWidth();
        this._renderShader();
        this._toggleMenuPositionClass()
    },
    _render: function() {
        this.callBase();
        this._dimensionChanged()
    },
    _renderMarkup: function() {
        var $wrapper = (0, _renderer2.default)("<div>").addClass(DRAWER_WRAPPER_CLASS);
        this._$menu = (0, _renderer2.default)("<div>").addClass(DRAWER_MENU_CONTENT_CLASS);
        this._$container = (0, _renderer2.default)("<div>").addClass(DRAWER_CONTENT_CLASS);
        $wrapper.append(this._$menu);
        $wrapper.append(this._$container);
        this.$element().append($wrapper)
    },
    _refreshModeClass: function(prevClass) {
        prevClass && this.$element().removeClass(DRAWER_CLASS + "-" + prevClass);
        this.$element().addClass(DRAWER_CLASS + "-" + this.option("mode"))
    },
    _refreshShowModeClass: function(prevClass) {
        prevClass && this.$element().removeClass(DRAWER_CLASS + "-" + prevClass);
        this.$element().addClass(DRAWER_CLASS + "-" + this.option("showMode"))
    },
    _renderShader: function() {
        this._$shader = this._$shader || (0, _renderer2.default)("<div>").addClass(DRAWER_SHADER_CLASS);
        this._$shader.appendTo(this.content());
        _events_engine2.default.off(this._$shader, _click2.default.name);
        _events_engine2.default.on(this._$shader, _click2.default.name, this.hideMenu.bind(this));
        this._toggleShaderVisibility(this.option("menuVisible"))
    },
    _initWidth: function() {
        this._minWidth = this.option("minWidth") || 0;
        this._maxWidth = this.option("maxWidth") || this.getRealMenuWidth()
    },
    getMaxWidth: function() {
        return this._maxWidth
    },
    getMinWidth: function() {
        return this._minWidth
    },
    getRealMenuWidth: function() {
        var $menu = this._$menu;
        return $menu.get(0).hasChildNodes() ? $menu.get(0).childNodes[0].getBoundingClientRect().width : $menu.get(0).getBoundingClientRect().width
    },
    _isRightMenuPosition: function() {
        var invertedPosition = "inverted" === this.option("menuPosition");
        var rtl = this.option("rtlEnabled");
        return rtl && !invertedPosition || !rtl && invertedPosition
    },
    _toggleMenuPositionClass: function() {
        var menuPosition = this.option("menuPosition");
        this._$menu.removeClass(DRAWER_CLASS + "-left");
        this._$menu.removeClass(DRAWER_CLASS + "-right");
        this._$menu.removeClass(DRAWER_CLASS + "-top");
        this._$menu.addClass(DRAWER_CLASS + "-" + menuPosition)
    },
    _renderPosition: function(offset, animate) {
        this._animations = [];
        animate = _type2.default.isDefined(animate) ? animate && this.option("animationEnabled") : this.option("animationEnabled");
        if (!_window2.default.hasWindow()) {
            return
        }
        var duration = this.option("animationDuration");
        this._toggleHideMenuCallback(offset);
        offset && this._toggleShaderVisibility(offset);
        this._strategy.renderPosition(offset, animate);
        this._strategy.renderShaderVisibility(offset, animate, duration)
    },
    _animationCompleteHandler: function() {
        if (this._deferredAnimate) {
            this._deferredAnimate.resolveWith(this);
            this._animations = []
        }
    },
    _toggleHideMenuCallback: function(subscribe) {
        if (subscribe) {
            _hide_top_overlay.hideCallback.add(this._hideMenuHandler)
        } else {
            _hide_top_overlay.hideCallback.remove(this._hideMenuHandler)
        }
    },
    _getRTLSignCorrection: function() {
        return this._isRightMenuPosition() ? -1 : 1
    },
    _dispose: function() {
        _uiDrawerRendering.animation.complete((0, _renderer2.default)(this.content()));
        this._toggleHideMenuCallback(false);
        this.callBase()
    },
    _visibilityChanged: function(visible) {
        if (visible) {
            this._dimensionChanged()
        }
    },
    _dimensionChanged: function() {
        delete this._menuWidth;
        this._renderPosition(this.option("menuVisible"), false)
    },
    _toggleShaderVisibility: function(visible) {
        if (this.option("showShader")) {
            this._$shader.toggleClass(INVISIBLE_STATE_CLASS, !visible)
        } else {
            this._$shader.toggleClass(INVISIBLE_STATE_CLASS, true)
        }
    },
    _togglePositionClass: function(menuVisible) {
        this.$element().toggleClass(OPENED_STATE_CLASS, menuVisible)
    },
    _optionChanged: function(args) {
        switch (args.name) {
            case "width":
                this.callBase(args);
                this._dimensionChanged();
                break;
            case "contentOffset":
                this._dimensionChanged();
                break;
            case "menuVisible":
                this._renderPosition(args.value);
                this._togglePositionClass(args.value);
                break;
            case "menuPosition":
                this._toggleMenuPositionClass();
                this._renderPosition(this.option("menuVisible"));
                break;
            case "contentTemplate":
            case "menuTemplate":
                this._invalidate();
                break;
            case "mode":
                this._initStrategy();
                _translator2.default.move(this._$menu, {
                    left: 0
                });
                this._refreshModeClass(args.previousValue);
                this._renderPosition(this.option("menuVisible"));
                this.repaint();
                break;
            case "minWidth":
            case "maxWidth":
                this._initWidth();
                _translator2.default.move(this._$menu, {
                    left: 0
                });
                this._renderPosition(this.option("menuVisible"));
                this.repaint();
                break;
            case "showMode":
                this._refreshShowModeClass(args.previousValue);
                this.repaint();
                break;
            case "showShader":
                this._refreshModeClass(args.previousValue);
                break;
            case "animationEnabled":
            case "animationDuration":
                break;
            default:
                this.callBase(args)
        }
    },
    menuContent: function() {
        return (0, _dom.getPublicElement)(this._$menu)
    },
    content: function() {
        return (0, _dom.getPublicElement)(this._$container)
    },
    showMenu: function() {
        return this.toggleMenuVisibility(true)
    },
    hideMenu: function() {
        return this.toggleMenuVisibility(false)
    },
    toggleMenuVisibility: function(showing) {
        showing = void 0 === showing ? !this.option("menuVisible") : showing;
        this._deferredAnimate = new _deferred.Deferred;
        this.option("menuVisible", showing);
        return this._deferredAnimate.promise()
    }
});
(0, _component_registrator2.default)("dxDrawer", Drawer);
module.exports = Drawer;
