/**
 * DevExtreme (ui/drawer/ui.drawer.rendering.strategy.js)
 * Version: 18.2.0 (build 18207)
 * Build date: Thu Jul 26 2018
 *
 * Copyright (c) 2012 - 2018 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
var _class = require("../../core/class");
var _class2 = _interopRequireDefault(_class);
var _deferred = require("../../core/utils/deferred");
var _deferred2 = _interopRequireDefault(_deferred);
var _renderer = require("../../core/renderer");
var _renderer2 = _interopRequireDefault(_renderer);
var _fx = require("../../animation/fx");
var _fx2 = _interopRequireDefault(_fx);

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        "default": obj
    }
}
var when = _deferred2.default.when;
var animation = {
    moveTo: function($element, position, duration, completeAction) {
        _fx2.default.animate($element, {
            type: "slide",
            to: {
                left: position
            },
            duration: duration,
            complete: completeAction
        })
    },
    paddingLeft: function($element, padding, duration, completeAction) {
        var toConfig = {};
        toConfig["padding-left"] = padding;
        _fx2.default.animate($element, {
            to: {
                paddingLeft: padding
            },
            duration: duration,
            complete: completeAction
        })
    },
    fade: function($element, config, duration, completeAction) {
        _fx2.default.animate($element, {
            type: "fade",
            to: config.to,
            from: config.from,
            duration: duration,
            complete: completeAction
        })
    },
    width: function($element, _width, duration, completeAction) {
        var toConfig = {};
        toConfig.width = _width;
        _fx2.default.animate($element, {
            to: toConfig,
            duration: duration,
            complete: completeAction
        })
    },
    complete: function($element) {
        _fx2.default.stop($element, true)
    }
};
var DrawerStrategy = _class2.default.inherit({
    ctor: function(drawer) {
        this._drawer = drawer
    },
    renderPosition: function(offset, animate) {
        this._contentAnimation = new _deferred.Deferred, this._menuAnimation = new _deferred.Deferred;
        this._shaderAnimation = new _deferred.Deferred;
        if (animate) {
            this._drawer._animations.push(this._contentAnimation);
            this._drawer._animations.push(this._menuAnimation);
            this._drawer._animations.push(this._shaderAnimation);
            when.apply(_renderer2.default, this._drawer._animations).done(function() {
                this._animationCompleteHandler()
            }.bind(this._drawer))
        }
    },
    _getMenuOffset: function(offset) {
        if (offset) {
            return -(this._drawer.getRealMenuWidth() - this._drawer.getMaxWidth())
        } else {
            return -(this._drawer.getRealMenuWidth() - this._drawer.getMinWidth())
        }
    },
    _getMenuWidth: function(offset) {
        return offset ? this._drawer.getMaxWidth() : this._drawer.getMinWidth()
    },
    renderShaderVisibility: function(offset, animate, duration) {
        var _this = this;
        var fadeConfig = this._getFadeConfig(offset);
        if (animate) {
            animation.fade((0, _renderer2.default)(this._drawer._$shader), fadeConfig, duration, function() {
                _this._shaderAnimation.resolve()
            })
        } else {
            this._drawer._$shader.css("opacity", fadeConfig.to)
        }
    },
    _getFadeConfig: function(offset) {
        if (offset) {
            return {
                to: .5,
                from: 0
            }
        } else {
            return {
                to: 0,
                from: .5
            }
        }
    }
});
module.exports = DrawerStrategy;
module.exports.animation = animation;
